#pragma once

#include <cstddef>
#include <cstdint>
#include <map>
#include <optional>
#include <spb/json.hpp>
#include <spb/pb.hpp>
#include <string>
#include <vector>

namespace libcore
{
struct ConnectionMetaData;

struct EmptyReq
{
};
struct EmptyResp
{
};
struct ErrorResp
{
std::optional<std::string> error = "";
};
struct LoadConfigReq
{
std::string core_config;
bool disable_stats;
std::optional<bool> need_extra_process = false;
std::optional<std::string> extra_process_path = "";
std::optional<std::string> extra_process_args = "";
std::optional<std::string> extra_process_conf = "";
std::optional<std::string> extra_process_conf_dir = "";
std::optional<bool> extra_no_out = false;
};
struct URLTestResp
{
std::string outbound_tag;
int32_t latency_ms;
std::string error;
};
struct TestReq
{
std::string config;
std::vector<std::string> outbound_tags;
bool use_default_outbound;
std::string url;
bool test_current;
int32_t max_concurrency;
};
struct TestResp
{
std::vector<URLTestResp> results;
};
struct QueryStatsResp
{
std::map< std::string, int64_t > ups;
std::map< std::string, int64_t > downs;
};
struct ListConnectionsResp
{
std::vector<ConnectionMetaData> connections;
};
struct ConnectionMetaData
{
std::string id;
int64_t created_at;
int64_t upload;
int64_t download;
std::string outbound;
std::string network;
std::string dest;
std::string protocol;
std::string domain;
std::string process;
};
struct GetGeoIPListResponse
{
std::vector<std::string> items;
};
struct GetGeoSiteListResponse
{
std::vector<std::string> items;
};
struct GeoListRequest
{
std::string path;
};
struct CompileGeoIPToSrsRequest
{
std::string item;
std::string path;
};
struct CompileGeoSiteToSrsRequest
{
std::string item;
std::string path;
};
struct SetSystemDNSRequest
{
bool clear;
};
struct IsPrivilegedResponse
{
bool has_privilege;
};
struct SpeedTestRequest
{
std::string config;
std::vector<std::string> outbound_tags;
bool test_current;
bool use_default_outbound;
bool test_download;
bool test_upload;
bool simple_download;
std::string simple_download_addr;
};
struct SpeedTestResult
{
std::string dl_speed;
std::string ul_speed;
int32_t latency;
std::string outbound_tag;
std::string error;
std::string server_name;
std::string server_country;
bool cancelled;
};
struct SpeedTestResponse
{
std::vector<SpeedTestResult> results;
};
struct QuerySpeedTestResponse
{
SpeedTestResult result;
bool is_running;
};
struct QueryURLTestResponse
{
std::vector<URLTestResp> results;
};
}// namespace libcore

namespace spb::json::detail
{
struct ostream;
struct istream;

void serialize_value( ostream & stream, const libcore::EmptyReq & value );
void deserialize_value( istream & stream, libcore::EmptyReq & value );


void serialize_value( ostream & stream, const libcore::EmptyResp & value );
void deserialize_value( istream & stream, libcore::EmptyResp & value );


void serialize_value( ostream & stream, const libcore::ErrorResp & value );
void deserialize_value( istream & stream, libcore::ErrorResp & value );


void serialize_value( ostream & stream, const libcore::LoadConfigReq & value );
void deserialize_value( istream & stream, libcore::LoadConfigReq & value );


void serialize_value( ostream & stream, const libcore::URLTestResp & value );
void deserialize_value( istream & stream, libcore::URLTestResp & value );


void serialize_value( ostream & stream, const libcore::TestReq & value );
void deserialize_value( istream & stream, libcore::TestReq & value );


void serialize_value( ostream & stream, const libcore::TestResp & value );
void deserialize_value( istream & stream, libcore::TestResp & value );


void serialize_value( ostream & stream, const libcore::QueryStatsResp & value );
void deserialize_value( istream & stream, libcore::QueryStatsResp & value );


void serialize_value( ostream & stream, const libcore::ListConnectionsResp & value );
void deserialize_value( istream & stream, libcore::ListConnectionsResp & value );


void serialize_value( ostream & stream, const libcore::ConnectionMetaData & value );
void deserialize_value( istream & stream, libcore::ConnectionMetaData & value );


void serialize_value( ostream & stream, const libcore::GetGeoIPListResponse & value );
void deserialize_value( istream & stream, libcore::GetGeoIPListResponse & value );


void serialize_value( ostream & stream, const libcore::GetGeoSiteListResponse & value );
void deserialize_value( istream & stream, libcore::GetGeoSiteListResponse & value );


void serialize_value( ostream & stream, const libcore::GeoListRequest & value );
void deserialize_value( istream & stream, libcore::GeoListRequest & value );


void serialize_value( ostream & stream, const libcore::CompileGeoIPToSrsRequest & value );
void deserialize_value( istream & stream, libcore::CompileGeoIPToSrsRequest & value );


void serialize_value( ostream & stream, const libcore::CompileGeoSiteToSrsRequest & value );
void deserialize_value( istream & stream, libcore::CompileGeoSiteToSrsRequest & value );


void serialize_value( ostream & stream, const libcore::SetSystemDNSRequest & value );
void deserialize_value( istream & stream, libcore::SetSystemDNSRequest & value );


void serialize_value( ostream & stream, const libcore::IsPrivilegedResponse & value );
void deserialize_value( istream & stream, libcore::IsPrivilegedResponse & value );


void serialize_value( ostream & stream, const libcore::SpeedTestRequest & value );
void deserialize_value( istream & stream, libcore::SpeedTestRequest & value );


void serialize_value( ostream & stream, const libcore::SpeedTestResult & value );
void deserialize_value( istream & stream, libcore::SpeedTestResult & value );


void serialize_value( ostream & stream, const libcore::SpeedTestResponse & value );
void deserialize_value( istream & stream, libcore::SpeedTestResponse & value );


void serialize_value( ostream & stream, const libcore::QuerySpeedTestResponse & value );
void deserialize_value( istream & stream, libcore::QuerySpeedTestResponse & value );


void serialize_value( ostream & stream, const libcore::QueryURLTestResponse & value );
void deserialize_value( istream & stream, libcore::QueryURLTestResponse & value );

} // namespace spb::json::detail
namespace spb::pb::detail
{
struct ostream;
struct istream;

void serialize( ostream & stream, const libcore::EmptyReq & value );
void deserialize_value( istream & stream, libcore::EmptyReq & value, uint32_t tag );


void serialize( ostream & stream, const libcore::EmptyResp & value );
void deserialize_value( istream & stream, libcore::EmptyResp & value, uint32_t tag );


void serialize( ostream & stream, const libcore::ErrorResp & value );
void deserialize_value( istream & stream, libcore::ErrorResp & value, uint32_t tag );


void serialize( ostream & stream, const libcore::LoadConfigReq & value );
void deserialize_value( istream & stream, libcore::LoadConfigReq & value, uint32_t tag );


void serialize( ostream & stream, const libcore::URLTestResp & value );
void deserialize_value( istream & stream, libcore::URLTestResp & value, uint32_t tag );


void serialize( ostream & stream, const libcore::TestReq & value );
void deserialize_value( istream & stream, libcore::TestReq & value, uint32_t tag );


void serialize( ostream & stream, const libcore::TestResp & value );
void deserialize_value( istream & stream, libcore::TestResp & value, uint32_t tag );


void serialize( ostream & stream, const libcore::QueryStatsResp & value );
void deserialize_value( istream & stream, libcore::QueryStatsResp & value, uint32_t tag );


void serialize( ostream & stream, const libcore::ListConnectionsResp & value );
void deserialize_value( istream & stream, libcore::ListConnectionsResp & value, uint32_t tag );


void serialize( ostream & stream, const libcore::ConnectionMetaData & value );
void deserialize_value( istream & stream, libcore::ConnectionMetaData & value, uint32_t tag );


void serialize( ostream & stream, const libcore::GetGeoIPListResponse & value );
void deserialize_value( istream & stream, libcore::GetGeoIPListResponse & value, uint32_t tag );


void serialize( ostream & stream, const libcore::GetGeoSiteListResponse & value );
void deserialize_value( istream & stream, libcore::GetGeoSiteListResponse & value, uint32_t tag );


void serialize( ostream & stream, const libcore::GeoListRequest & value );
void deserialize_value( istream & stream, libcore::GeoListRequest & value, uint32_t tag );


void serialize( ostream & stream, const libcore::CompileGeoIPToSrsRequest & value );
void deserialize_value( istream & stream, libcore::CompileGeoIPToSrsRequest & value, uint32_t tag );


void serialize( ostream & stream, const libcore::CompileGeoSiteToSrsRequest & value );
void deserialize_value( istream & stream, libcore::CompileGeoSiteToSrsRequest & value, uint32_t tag );


void serialize( ostream & stream, const libcore::SetSystemDNSRequest & value );
void deserialize_value( istream & stream, libcore::SetSystemDNSRequest & value, uint32_t tag );


void serialize( ostream & stream, const libcore::IsPrivilegedResponse & value );
void deserialize_value( istream & stream, libcore::IsPrivilegedResponse & value, uint32_t tag );


void serialize( ostream & stream, const libcore::SpeedTestRequest & value );
void deserialize_value( istream & stream, libcore::SpeedTestRequest & value, uint32_t tag );


void serialize( ostream & stream, const libcore::SpeedTestResult & value );
void deserialize_value( istream & stream, libcore::SpeedTestResult & value, uint32_t tag );


void serialize( ostream & stream, const libcore::SpeedTestResponse & value );
void deserialize_value( istream & stream, libcore::SpeedTestResponse & value, uint32_t tag );


void serialize( ostream & stream, const libcore::QuerySpeedTestResponse & value );
void deserialize_value( istream & stream, libcore::QuerySpeedTestResponse & value, uint32_t tag );


void serialize( ostream & stream, const libcore::QueryURLTestResponse & value );
void deserialize_value( istream & stream, libcore::QueryURLTestResponse & value, uint32_t tag );

} // namespace spb::pb::detail
